@@ -5,9 +5,9 @@ import ssl
 from contextlib import AbstractContextManager, ExitStack
 from threading import Thread
 from typing import NoReturn
+from unittest import mock
 
 import pytest
-from pytest_mock import MockerFixture
 from trustme import CA
 
 from anyio import (
@@ -20,9 +20,15 @@ from anyio import (
     create_tcp_listener,
     to_thread,
 )
-from anyio.abc import AnyByteStream, SocketAttribute, SocketStream
+from anyio.abc import (
+    AnyByteStream,
+    ObjectStream,
+    ObjectStreamConnectable,
+    SocketAttribute,
+    SocketStream,
+)
 from anyio.streams.stapled import StapledObjectStream
-from anyio.streams.tls import TLSAttribute, TLSListener, TLSStream
+from anyio.streams.tls import TLSAttribute, TLSConnectable, TLSListener, TLSStream
 
 pytestmark = pytest.mark.anyio
 
@@ -375,18 +381,44 @@ class TestTLSStream:
         not hasattr(ssl, "OP_IGNORE_UNEXPECTED_EOF"),
         reason="The ssl module does not have the OP_IGNORE_UNEXPECTED_EOF attribute",
     )
-    async def test_default_context_ignore_unexpected_eof_flag_off(
-        self, mocker: MockerFixture
-    ) -> None:
+    async def test_default_context_ignore_unexpected_eof_flag_off(self) -> None:
         send1, receive1 = create_memory_object_stream[bytes]()
         client_stream = StapledObjectStream(send1, receive1)
-        mocker.patch.object(TLSStream, "_call_sslobject_method")
-        tls_stream = await TLSStream.wrap(client_stream)
-        ssl_context = tls_stream.extra(TLSAttribute.ssl_object).context
-        assert not ssl_context.options & ssl.OP_IGNORE_UNEXPECTED_EOF
+        with mock.patch.object(TLSStream, "_call_sslobject_method"):
+            tls_stream = await TLSStream.wrap(client_stream)
+            ssl_context = tls_stream.extra(TLSAttribute.ssl_object).context
+            assert not ssl_context.options & ssl.OP_IGNORE_UNEXPECTED_EOF
 
-        send1.close()
-        receive1.close()
+            send1.close()
+            receive1.close()
+
+    async def test_truststore_ssl(
+        self, request: pytest.FixtureRequest, server_context: ssl.SSLContext
+    ) -> None:
+        # This test is only expected to fail on Windows without the associated patch
+        def serve_sync() -> None:
+            with server_sock, pytest.raises(ssl.SSLEOFError):
+                server_sock.accept()
+
+        # We deliberately skip making the client context trust the server context
+        truststore = pytest.importorskip("truststore")
+        client_context = truststore.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
+
+        server_sock = server_context.wrap_socket(
+            socket.socket(), server_side=True, suppress_ragged_eofs=True
+        )
+        server_sock.settimeout(1)
+        server_sock.bind(("127.0.0.1", 0))
+        server_sock.listen()
+        server_thread = Thread(target=serve_sync, daemon=True)
+        server_thread.start()
+        request.addfinalizer(server_thread.join)
+
+        async with await connect_tcp(*server_sock.getsockname()) as stream:
+            with pytest.raises(ssl.SSLCertVerificationError):
+                await TLSStream.wrap(
+                    stream, hostname="localhost", ssl_context=client_context
+                )
 
 
 class TestTLSListener:
@@ -499,3 +531,35 @@ class TestTLSListener:
             await event.wait()
             await to_thread.run_sync(client_thread.join)
             tg.cancel_scope.cancel()
+
+
+async def test_tls_connectable(
+    client_context: ssl.SSLContext, server_context: ssl.SSLContext
+) -> None:
+    server_send, server_receive = create_memory_object_stream[bytes](1)
+    client_send, client_receive = create_memory_object_stream[bytes](1)
+    client_stream = StapledObjectStream(client_send, server_receive)
+    server_stream = StapledObjectStream(server_send, client_receive)
+
+    async def server() -> None:
+        async with await TLSStream.wrap(
+            server_stream,
+            server_side=True,
+            hostname="localhost",
+            ssl_context=server_context,
+        ) as server_tls_stream:
+            message = await server_tls_stream.receive()
+            await server_tls_stream.send(b"hello, " + message)
+
+    class MemoryConnectable(ObjectStreamConnectable[bytes]):
+        async def connect(self) -> ObjectStream[bytes]:
+            return client_stream
+
+    async with create_task_group() as tg:
+        tg.start_soon(server)
+        connectable = TLSConnectable(
+            MemoryConnectable(), ssl_context=client_context, hostname="localhost"
+        )
+        async with await connectable.connect() as client_tls_stream:
+            await client_tls_stream.send(b"world")
+            assert await client_tls_stream.receive() == b"hello, world"
