@@ -1,6 +1,9 @@
+from __future__ import annotations
+
 import os
 import signal
 import sys
+from collections.abc import AsyncIterable
 
 import pytest
 
@@ -9,25 +12,30 @@ from anyio import create_task_group, fail_after, open_signal_receiver, to_thread
 pytestmark = [
     pytest.mark.anyio,
     pytest.mark.skipif(
-        sys.platform == 'win32',
-        reason='Signal delivery cannot be tested on Windows',
+        sys.platform == "win32",
+        reason="Signal delivery cannot be tested on Windows",
     ),
 ]
 
 
-async def test_receive_signals():
+async def test_receive_signals() -> None:
     with open_signal_receiver(signal.SIGUSR1, signal.SIGUSR2) as sigiter:
         await to_thread.run_sync(os.kill, os.getpid(), signal.SIGUSR1)
         await to_thread.run_sync(os.kill, os.getpid(), signal.SIGUSR2)
         with fail_after(1):
-            assert await sigiter.__anext__() == signal.SIGUSR1
-            assert await sigiter.__anext__() == signal.SIGUSR2
+            sigusr1 = await sigiter.__anext__()
+            assert isinstance(sigusr1, signal.Signals)
+            assert sigusr1 == signal.Signals.SIGUSR1
+
+            sigusr2 = await sigiter.__anext__()
+            assert isinstance(sigusr2, signal.Signals)
+            assert sigusr2 == signal.Signals.SIGUSR2
 
 
-async def test_task_group_cancellation_open():
-    async def signal_handler():
+async def test_task_group_cancellation_open() -> None:
+    async def signal_handler() -> None:
         with open_signal_receiver(signal.SIGUSR1) as sigiter:
-            async for v in sigiter:
+            async for _ in sigiter:
                 pytest.fail("SIGUSR1 should not be sent")
 
             pytest.fail("signal_handler should have been cancelled")
@@ -39,9 +47,9 @@ async def test_task_group_cancellation_open():
         tg.cancel_scope.cancel()
 
 
-async def test_task_group_cancellation_consume():
-    async def consume(sigiter):
-        async for v in sigiter:
+async def test_task_group_cancellation_consume() -> None:
+    async def consume(sigiter: AsyncIterable[int]) -> None:
+        async for _ in sigiter:
             pytest.fail("SIGUSR1 should not be sent")
 
         pytest.fail("consume should have been cancelled")
