@@ -4,7 +4,6 @@ import os
 import platform
 import sys
 from collections.abc import Callable
-from contextlib import ExitStack
 from pathlib import Path
 from subprocess import CalledProcessError
 from textwrap import dedent
@@ -130,14 +129,16 @@ async def test_process_new_session_sid() -> None:
     assert result.stdout.decode().strip() != str(sid)
 
 
-async def test_run_process_connect_to_file(tmp_path: Path) -> None:
+async def test_open_process_connect_to_file(tmp_path: Path) -> None:
     stdinfile = tmp_path / "stdin"
     stdinfile.write_text("Hello, process!\n")
     stdoutfile = tmp_path / "stdout"
     stderrfile = tmp_path / "stderr"
-    with stdinfile.open("rb") as fin, stdoutfile.open("wb") as fout, stderrfile.open(
-        "wb"
-    ) as ferr:
+    with (
+        stdinfile.open("rb") as fin,
+        stdoutfile.open("wb") as fout,
+        stderrfile.open("wb") as ferr,
+    ):
         async with await open_process(
             [
                 sys.executable,
@@ -161,13 +162,51 @@ async def test_run_process_connect_to_file(tmp_path: Path) -> None:
     )
 
 
+async def test_run_process_connect_to_file(tmp_path: Path) -> None:
+    stdinfile = tmp_path / "stdin"
+    stdinfile.write_text("Hello, process!\n")
+    stdoutfile = tmp_path / "stdout"
+    stderrfile = tmp_path / "stderr"
+    with (
+        stdinfile.open("rb") as fin,
+        stdoutfile.open("wb") as fout,
+        stderrfile.open("wb") as ferr,
+    ):
+        await run_process(
+            [
+                sys.executable,
+                "-c",
+                "import sys; txt = sys.stdin.read().strip(); "
+                'print("stdin says", repr(txt), "but stderr says NO!", '
+                "file=sys.stderr); "
+                'print("stdin says", repr(txt), "and stdout says YES!")',
+            ],
+            stdin=fin,
+            stdout=fout,
+            stderr=ferr,
+        )
+
+    assert (
+        stdoutfile.read_text() == "stdin says 'Hello, process!' and stdout says YES!\n"
+    )
+    assert (
+        stderrfile.read_text() == "stdin says 'Hello, process!' but stderr says NO!\n"
+    )
+
+
+async def test_stdin_input_both_passed(tmp_path: Path) -> None:
+    stdinfile = tmp_path / "stdin"
+    stdinfile.write_text("Hello, process!\n")
+    with pytest.raises(ValueError, match="only one of"), stdinfile.open("rb") as fin:
+        await run_process([sys.executable, "--version"], input=b"abc", stdin=fin)
+
+
 async def test_run_process_inherit_stdout(capfd: pytest.CaptureFixture[str]) -> None:
     await run_process(
         [
             sys.executable,
             "-c",
-            'import sys; print("stderr-text", file=sys.stderr); '
-            'print("stdout-text")',
+            'import sys; print("stderr-text", file=sys.stderr); print("stdout-text")',
         ],
         check=True,
         stdout=None,
@@ -271,30 +310,21 @@ async def test_py39_arguments(
     anyio_backend_name: str,
     anyio_backend_options: dict[str, Any],
 ) -> None:
-    with ExitStack() as stack:
-        if sys.version_info < (3, 9):
-            stack.enter_context(
-                pytest.raises(
-                    TypeError,
-                    match=rf"the {argname!r} argument requires Python 3.9 or later",
-                )
-            )
+    try:
+        await run_process(
+            [sys.executable, "-c", "print('hello')"],
+            **{argname: argvalue_factory()},
+        )
+    except ValueError as exc:
+        if (
+            "unexpected kwargs" in str(exc)
+            and anyio_backend_name == "asyncio"
+            and anyio_backend_options["loop_factory"]
+            and anyio_backend_options["loop_factory"].__module__ == "uvloop"
+        ):
+            pytest.skip(f"the {argname!r} argument is not supported by uvloop yet")
 
-        try:
-            await run_process(
-                [sys.executable, "-c", "print('hello')"],
-                **{argname: argvalue_factory()},
-            )
-        except ValueError as exc:
-            if (
-                "unexpected kwargs" in str(exc)
-                and anyio_backend_name == "asyncio"
-                and anyio_backend_options["loop_factory"]
-                and anyio_backend_options["loop_factory"].__module__ == "uvloop"
-            ):
-                pytest.skip(f"the {argname!r} argument is not supported by uvloop yet")
-
-            raise
+        raise
 
 
 async def test_close_early() -> None:
@@ -316,9 +346,10 @@ async def test_close_while_reading() -> None:
     time.sleep(3)
     """)
 
-    async with await open_process(
-        [sys.executable, "-c", code]
-    ) as process, create_task_group() as tg:
+    async with (
+        await open_process([sys.executable, "-c", code]) as process,
+        create_task_group() as tg,
+    ):
         assert process.stdout
         tg.start_soon(process.stdout.aclose)
         with pytest.raises(ClosedResourceError):
